/**********************************************************************
 Copyright (c) 2006 Thomas Marti, Stefan Schmid and others. All rights reserved.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 Contributors:
 ...
 **********************************************************************/
package org.datanucleus.store.rdbms.mapping.jts2oracle;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import oracle.jdbc.OracleConnection;
import oracle.sql.STRUCT;

import org.datanucleus.store.rdbms.schema.OracleTypeInfo;
import org.datanucleus.store.rdbms.schema.SQLTypeInfo;
import org.datanucleus.store.rdbms.table.Column;
import org.datanucleus.exceptions.NucleusDataStoreException;
import org.datanucleus.store.mapped.DatastoreField;
import org.datanucleus.store.mapped.MappedStoreManager;
import org.datanucleus.store.mapped.mapping.JavaTypeMapping;
import org.datanucleus.store.rdbms.RDBMSStoreManager;
import org.datanucleus.store.rdbms.mapping.RDBMSMapping;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.io.oracle.OraReader;
import com.vividsolutions.jts.io.oracle.OraWriter;

/**
 * Mapping for a JTS Geometry object to Oracle.
 */
public class GeometryRDBMSMapping extends RDBMSMapping
{

    public GeometryRDBMSMapping(JavaTypeMapping mapping, MappedStoreManager storeMgr, DatastoreField field)
    {
        super(storeMgr, mapping);
        column = (Column) field;
        initialize();
    }

    public GeometryRDBMSMapping(MappedStoreManager storeMgr, JavaTypeMapping mapping)
    {
        super(storeMgr, mapping);
        initialize();
    }

    private void initialize()
    {
        initTypeInfo();
    }

    public SQLTypeInfo getTypeInfo() 
    {
        return ((RDBMSStoreManager)storeMgr).getSQLTypeInfoForJDBCType(OracleTypeInfo.TYPES_SDO_GEOMETRY);
    }

    public Object getObject(Object rs, int exprIndex)
    {
        Object value;

        try
        {
            Object st = ((ResultSet) rs).getObject(exprIndex);
            if (((ResultSet) rs).wasNull() || st == null)
            {
                value = null;
            }
            else
            {
                value = new OraReader().read((STRUCT)st);
            }
        }
        catch (SQLException e)
        {
            throw new NucleusDataStoreException(LOCALISER.msg("RDBMS.Mapping.UnableToGetParam", "Object", "" + exprIndex, column, e.getMessage()), e);
        }

        return value;
    }

    public void setObject(Object preparedStatement, int exprIndex, Object value)
    {
        PreparedStatement ps = (PreparedStatement)preparedStatement;
        try
        {
            if (value == null)
            {
                ps.setNull(exprIndex, getTypeInfo().getDataType(), getTypeInfo().getTypeName());
            }
            else
            {
                OracleConnection conn = (OracleConnection)ps.getConnection();
                Object obj = new OraWriter(conn).write((Geometry)value);
                ps.setObject(exprIndex, obj);
            }
        }
        catch (SQLException e)
        {
            throw new NucleusDataStoreException(LOCALISER.msg("RDBMS.Mapping.UnableToSetParam", "Object", "" + value, column, e.getMessage()),e);
        }
    }
}